@@ -1,6 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright by The HDF Group.                                               *
- * Copyright by the Board of Trustees of the University of Illinois.         *
  * All rights reserved.                                                      *
  *                                                                           *
  * This file is part of HDF5.  The full HDF5 copyright notice, including     *
@@ -17,39 +16,6 @@
 #ifndef H5Dpublic_H
 #define H5Dpublic_H
 
-/**\defgroup H5D H5D
- *
- * Use the functions in this module to manage HDF5 datasets, including the
- * transfer of data between memory and disk and the description of dataset
- * properties. Datasets are used by other HDF5 APIs and referenced either by
- * name or by a handle. Such handles can be obtained by either creating or
- * opening the dataset.
- *
- * Typical stages in the HDF5 dataset life cycle are shown below in introductory
- * examples.
- *
- * <table>
- * <tr><th>Create</th><th>Read</th></tr>
- * <tr valign="top">
- *   <td>
- *   \snippet{lineno} H5D_examples.c create
- *   </td>
- *   <td>
- *   \snippet{lineno} H5D_examples.c read
- *   </td>
- * <tr><th>Update</th><th>Delete</th></tr>
- * <tr valign="top">
- *   <td>
- *   \snippet{lineno} H5D_examples.c update
- *   </td>
- *   <td>
- *   \snippet{lineno} H5D_examples.c delete
- *   </td>
- * </tr>
- * </table>
- *
- */
-
 /* System headers needed by this file */
 
 /* Public headers needed by this file */
@@ -63,18 +29,12 @@
 /* Macros used to "unset" chunk cache configuration parameters */
 #define H5D_CHUNK_CACHE_NSLOTS_DEFAULT ((size_t)-1)
 #define H5D_CHUNK_CACHE_NBYTES_DEFAULT ((size_t)-1)
-#define H5D_CHUNK_CACHE_W0_DEFAULT     (-1.0f)
+#define H5D_CHUNK_CACHE_W0_DEFAULT     (-1.0)
 
-/* Property names for H5LTDdirect_chunk_write */
-#define H5D_XFER_DIRECT_CHUNK_WRITE_FLAG_NAME     "direct_chunk_flag"
-#define H5D_XFER_DIRECT_CHUNK_WRITE_FILTERS_NAME  "direct_chunk_filters"
-#define H5D_XFER_DIRECT_CHUNK_WRITE_OFFSET_NAME   "direct_chunk_offset"
-#define H5D_XFER_DIRECT_CHUNK_WRITE_DATASIZE_NAME "direct_chunk_datasize"
-
-/* Property names for H5LTDdirect_chunk_read */
-#define H5D_XFER_DIRECT_CHUNK_READ_FLAG_NAME    "direct_chunk_read_flag"
-#define H5D_XFER_DIRECT_CHUNK_READ_OFFSET_NAME  "direct_chunk_read_offset"
-#define H5D_XFER_DIRECT_CHUNK_READ_FILTERS_NAME "direct_chunk_read_filters"
+/**
+ * Bit flags for the H5Pset_chunk_opts() and H5Pget_chunk_opts()
+ */
+#define H5D_CHUNK_DONT_FILTER_PARTIAL_CHUNKS (0x0002u)
 
 /*******************/
 /* Public Typedefs */
@@ -89,7 +49,8 @@ typedef enum H5D_layout_t {
     H5D_COMPACT      = 0,  /**< raw data is small (< 64KB) */
     H5D_CONTIGUOUS   = 1,  /**< contiguous layout */
     H5D_CHUNKED      = 2,  /**< chunked or tiled layout */
-    H5D_NLAYOUTS     = 3   /**< this one must be last! */
+    H5D_VIRTUAL      = 3,  /**< actual data is stored in other datasets */
+    H5D_NLAYOUTS     = 4   /**< this one must be last! */
 } H5D_layout_t;
 //! <!-- [H5D_layout_t_snip] -->
 
@@ -98,7 +59,14 @@ typedef enum H5D_layout_t {
  * Types of chunk index data structures
  */
 typedef enum H5D_chunk_index_t {
-    H5D_CHUNK_BTREE = 0, /**< v1 B-tree index (default)                */
+    H5D_CHUNK_IDX_BTREE = 0, /**< v1 B-tree index (default)                */
+    H5D_CHUNK_IDX_SINGLE =
+        1, /**< Single Chunk index (cur dims[]=max dims[]=chunk dims[]; filtered & non-filtered) */
+    H5D_CHUNK_IDX_NONE   = 2, /**< Implicit: No Index (#H5D_ALLOC_TIME_EARLY, non-filtered, fixed dims) */
+    H5D_CHUNK_IDX_FARRAY = 3, /**< Fixed array (for 0 unlimited dims)       */
+    H5D_CHUNK_IDX_EARRAY = 4, /**< Extensible array (for 1 unlimited dim)   */
+    H5D_CHUNK_IDX_BT2    = 5, /**< v2 B-tree index (for >1 unlimited dims)  */
+    H5D_CHUNK_IDX_NTYPES      /**< This one must be last!                   */
 } H5D_chunk_index_t;
 //! <!-- [H5D_chunk_index_t_snip] -->
 
@@ -152,16 +120,30 @@ typedef enum H5D_fill_value_t {
 } H5D_fill_value_t;
 //! <!-- [H5D_fill_value_t_snip] -->
 
-/********************/
-/* Public Variables */
-/********************/
+//! <!-- [H5D_vds_view_t_snip] -->
+/**
+ * Values for VDS bounds option
+ */
+typedef enum H5D_vds_view_t {
+    H5D_VDS_ERROR          = -1, /**< Error */
+    H5D_VDS_FIRST_MISSING  = 0,  /**< Include all data before the first missing mapped data */
+    H5D_VDS_LAST_AVAILABLE = 1   /**< Include all available mapped data */
+} H5D_vds_view_t;
+//! <!-- [H5D_vds_view_t_snip] -->
 
-/*********************/
-/* Public Prototypes */
-/*********************/
-#ifdef __cplusplus
-extern "C" {
-#endif
+//! <!-- [H5D_append_cb_t_snip] -->
+/**
+ * \brief Callback for H5Pset_append_flush()
+ *
+ * \dset_id{dataset_id}
+ * \param[in] cur_dims The current extent of the dataset's dimensions
+ * \param[in,out] op_data User context
+ *
+ * \return \herr_t
+ *
+ */
+typedef herr_t (*H5D_append_cb_t)(hid_t dataset_id, hsize_t *cur_dims, void *op_data);
+//! <!-- [H5D_append_cb_t_snip] -->
 
 //! <!-- [H5D_operator_t_snip] -->
 /**
@@ -239,6 +221,38 @@ typedef herr_t (*H5D_scatter_func_t)(const void **src_buf /*out*/, size_t *src_b
 typedef herr_t (*H5D_gather_func_t)(const void *dst_buf, size_t dst_buf_bytes_used, void *op_data);
 //! <!-- [H5D_gather_func_t_snip] -->
 
+//! <!-- [H5D_chunk_iter_op_t_snip] -->
+/**
+ * \brief Callback for H5Dchunk_iter()
+ *
+ * \param[in]     offset      Logical position of the chunk’s first element in units of dataset elements
+ * \param[in]     filter_mask Bitmask indicating the filters used when the chunk was written
+ * \param[in]     addr        Chunk address in the file
+ * \param[in]     size        Chunk size in bytes, 0 if the chunk does not exist
+ * \param[in,out] op_data     Pointer to any user-defined data associated with
+ *                            the operation.
+ * \returns \li Zero (#H5_ITER_CONT) causes the iterator to continue, returning
+ *              zero when all elements have been processed.
+ *          \li A positive value (#H5_ITER_STOP) causes the iterator to
+ *              immediately return that value, indicating short-circuit success.
+ *          \li A negative (#H5_ITER_ERROR) causes the iterator to immediately
+ *              return that value, indicating failure.
+ */
+typedef int (*H5D_chunk_iter_op_t)(const hsize_t *offset, unsigned filter_mask, haddr_t addr, hsize_t size,
+                                   void *op_data);
+//! <!-- [H5D_chunk_iter_op_t_snip] -->
+
+/********************/
+/* Public Variables */
+/********************/
+
+/*********************/
+/* Public Prototypes */
+/*********************/
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * --------------------------------------------------------------------------
  * \ingroup H5D
@@ -265,7 +279,7 @@ typedef herr_t (*H5D_gather_func_t)(const void *dst_buf, size_t dst_buf_bytes_us
  *
  *          \p loc_id may specify a file, group, dataset, named datatype,
  *          or attribute.  If an attribute, dataset, or named datatype is
- *          specified then the dataset will be created at the location
+ *          specified, then the dataset will be created at the location
  *          where the attribute, dataset, or named datatype is attached.
  *
  *          \p name may be either an absolute path in the file or a relative
@@ -276,7 +290,7 @@ typedef herr_t (*H5D_gather_func_t)(const void *dst_buf, size_t dst_buf_bytes_us
  *          file location where the dataset will be created, the datatype
  *          is copied and converted to a transient type.
  *
- *          The link creation property list, \p lcpl_id, governs creation
+ *          The link creation property list, \p lcpl_id, governs the creation
  *          of the link(s) by which the new dataset is accessed and the
  *          creation of any intermediate groups that may be missing.
  *
@@ -320,12 +334,12 @@ H5_DLL hid_t H5Dcreate2(hid_t loc_id, const char *name, hid_t type_id, hid_t spa
  *
  *          \p loc_id may specify a file, group, dataset, named datatype,
  *          or attribute.  If an attribute, dataset, or named datatype is
- *          specified then the dataset will be created at the location
+ *          specified, then the dataset will be created at the location
  *          where the attribute, dataset, or named datatype is attached.
  *
  *          The dataset’s datatype and dataspace are specified by
  *          \p type_id and \p space_id, respectively. These are the
- *          datatype and dataspace of the dataset as it will exist in
+ *          datatype and dataspace of the dataset as they will exist in
  *          the file, which may differ from the datatype and dataspace
  *          in application memory.
  *
@@ -345,7 +359,7 @@ H5_DLL hid_t H5Dcreate_anon(hid_t loc_id, hid_t type_id, hid_t space_id, hid_t d
  * --------------------------------------------------------------------------
  * \ingroup H5D
  *
- * \brief Creates a new dataset and links it into the file
+ * \brief Opens an existing dataset
  *
  * \fgdta_loc_id
  * \param[in] name      Name of the dataset to open
@@ -409,6 +423,20 @@ H5_DLL hid_t H5Dget_space(hid_t dset_id);
  * \details H5Dget_space_status() determines whether space has been allocated
  *          for the dataset \p dset_id.
  *
+ * \note \Bold{BUG:} Prior to the HDF5 1.14.0, 1.12.2 and 1.10.9 releases,
+ *       H5Dget_space_status() may return incorrect space allocation status
+ *       values for datasets with filters applied to them.
+ *       H5Dget_space_status() calculated the space allocation status by
+ *       comparing the sum of the sizes of all the allocated chunks in the
+ *       dataset against the total data size of the dataset, as calculated by
+ *       the number of elements in the dataset's dataspace multiplied by the
+ *       dataset's datatype size. If the dataset had any compression filters
+ *       applied to it and the dataset chunks were successfully compressed,
+ *       the sum of the sizes of the allocated dataset chunks would generally
+ *       always be less than the total data size of the dataset, and
+ *       H5Dget_space_status() wouldn't ever return
+ *       `H5D_SPACE_STATUS_ALLOCATED`.
+ *
  * \since 1.6.0
  *
  */
@@ -449,6 +477,9 @@ H5_DLL hid_t H5Dget_type(hid_t dset_id);
  *          a copy of the dataset creation property list associated with
  *          the dataset specified by \p dset_id.
  *
+ *          The creation property list identifier should be released with
+ *          H5Pclose() to prevent resource leaks.
+ *
  */
 H5_DLL hid_t H5Dget_create_plist(hid_t dset_id);
 
@@ -553,6 +584,149 @@ H5_DLL hsize_t H5Dget_storage_size(hid_t dset_id);
  */
 H5_DLL herr_t H5Dget_chunk_storage_size(hid_t dset_id, const hsize_t *offset, hsize_t *chunk_bytes);
 
+/**
+ * --------------------------------------------------------------------------
+ * \ingroup H5D
+ *
+ * \brief Retrieves number of chunks that have nonempty intersection with a
+ *        specified selection
+ *
+ * \dset_id
+ * \param[in]  fspace_id   File dataspace selection identifier
+ * \param[out] nchunks     Number of chunks in the selection
+ *
+ * \return \herr_t
+ *
+ * \details H5Dget_num_chunks() retrieves the number of chunks
+ *          nchunks in a set of selected elements specified by \p fspace_id
+ *          for a dataset specified by the identifier \p dset_id. If \p
+ *          fspace_id is #H5S_ALL, the function will retrieve the total
+ *          number of chunks stored for the dataset.
+ *
+ *          \p fspace_id specifies the file dataspace selection.  It is
+ *          intended to take #H5S_ALL for specifying the current selection.
+ *
+ *          \note Please be aware that this function currently does not
+ *          support non-trivial selections, thus \p fspace_id has no
+ *          effect. Also, the implementation does not handle the #H5S_ALL
+ *          macro correctly.  As a workaround, application can get
+ *          the dataspace for the dataset using H5Dget_space() and pass that
+ *          in for \p fspace_id.  This will be fixed in a future release.
+ *
+ * \since 1.10.5
+ *
+ */
+H5_DLL herr_t H5Dget_num_chunks(hid_t dset_id, hid_t fspace_id, hsize_t *nchunks);
+
+/**
+ * --------------------------------------------------------------------------
+ * \ingroup H5D
+ *
+ * \brief Retrieves information about a chunk specified by its coordinates
+ *
+ * \dset_id
+ * \param[in]  offset      Logical position of the chunk’s first element in units of dataset elements
+ * \param[out] filter_mask Bitmask indicating the filters used when the chunk was written
+ * \param[out] addr        Chunk address in the file
+ * \param[out] size        Chunk size in bytes, 0 if the chunk does not exist
+ *
+ * \return \herr_t
+ *
+ * \details H5Dget_chunk_info_by_coord() retrieves the \p filter_mask, \p size,
+ *          and \p addr for a chunk in the dataset specified by \p dset_id,
+ *          using the coordinates specified by \p offset.
+ *
+ *          If the queried chunk does not exist in the file, \p size will
+ *          be set to 0, \p addr to \c HADDR_UNDEF, and the buffer \p
+ *          filter_mask will not be modified.
+ *
+ *          \p offset is a pointer to a one-dimensional array with a size
+ *          equal to the dataset’s rank. Each element is the logical
+ *          position of the chunk’s first element in a dimension.
+ *
+ * \since 1.10.5
+ *
+ */
+H5_DLL herr_t H5Dget_chunk_info_by_coord(hid_t dset_id, const hsize_t *offset, unsigned *filter_mask,
+                                         haddr_t *addr, hsize_t *size);
+
+/**
+ * --------------------------------------------------------------------------
+ * \ingroup H5D
+ *
+ * \brief Iterate over all chunks of a chunked dataset
+ *
+ * \dset_id
+ * \param[in]  dxpl_id  Identifier of a transfer property list
+ * \param[in]  cb       User callback function, called for every chunk.
+ * \param[in]  op_data  User-defined pointer to data required by op
+ *
+ * \return \herr_t
+ *
+ * \details H5Dchunk_iter iterates over all chunks in the dataset, calling the
+ *          user supplied callback with the details of the chunk and the supplied
+ *          context \p op_data.
+ *
+ * \par Example
+ * For each chunk, print the allocated chunk size (0 for unallocated chunks).
+ * \snippet H5D_examples.c H5Dchunk_iter_cb
+ * Iterate over all chunked datasets and chunks in a file.
+ * \snippet H5D_examples.c H5Ovisit_cb
+ *
+ * \since 1.10.9
+ *
+ */
+H5_DLL herr_t H5Dchunk_iter(hid_t dset_id, hid_t dxpl_id, H5D_chunk_iter_op_t cb, void *op_data);
+
+/**
+ * --------------------------------------------------------------------------
+ * \ingroup H5D
+ *
+ * \brief Retrieves information about a chunk specified by its index
+ *
+ * \dset_id
+ * \param[in]  fspace_id File dataspace selection identifier (See Note below)
+ * \param[in]  chk_idx   Index of the chunk
+ * \param[out] offset    Logical position of the chunk’s first element in units of dataset elements
+ * \param[out] filter_mask Bitmask indicating the filters used when the chunk was written
+ * \param[out] addr      Chunk address in the file
+ * \param[out] size      Chunk size in bytes, 0 if the chunk does not exist
+ *
+ * \return \herr_t
+ *
+ * \details H5Dget_chunk_info() retrieves the offset coordinates,
+ *          \p offset, filter mask, \p filter_mask, size, \p size, and address
+ *          \p addr for the dataset specified by the identifier \p dset_id and the chunk
+ *          specified by the index \p index. The chunk belongs to a set of
+ *          chunks in the selection specified by \p fspace_id. If the queried
+ *          chunk does not exist in the file, the size will be set to 0 and
+ *          address to HADDR_UNDEF. The value pointed to by filter_mask will
+ *          not be modified. \c NULL can be passed in for any \p out parameters.
+ *
+ *          \p chk_idx is the chunk index in the selection. The index value
+ *          may have a value of 0 up to the number of chunks stored in
+ *          the file that has a nonempty intersection with the file
+ *          dataspace selection.
+ *
+ *          \note As of 1.10.5, the dataspace intersection is not yet
+ *          supported. Hence, the index is of all the written chunks.
+ *
+ *          \p fspace_id specifies the file dataspace selection.  It is
+ *          intended to take #H5S_ALL to specify the current selection.
+ *
+ *          \note Please be aware that this function currently does not
+ *          support non-trivial selections; thus \p fspace_id has no
+ *          effect. Also, the implementation does not handle the #H5S_ALL
+ *          macro correctly.  As a workaround, an application can get
+ *          the dataspace for the dataset using H5Dget_space() and pass that
+ *          in for \p fspace_id.  This will be fixed in a future release.
+ *
+ * \since 1.10.5
+ *
+ */
+H5_DLL herr_t H5Dget_chunk_info(hid_t dset_id, hid_t fspace_id, hsize_t chk_idx, hsize_t *offset,
+                                unsigned *filter_mask, haddr_t *addr, hsize_t *size);
+
 /**
  * --------------------------------------------------------------------------
  * \ingroup H5D
@@ -561,7 +735,7 @@ H5_DLL herr_t H5Dget_chunk_storage_size(hid_t dset_id, const hsize_t *offset, hs
  *
  * \dset_id
  *
- * \return Returns the offset in bytes; otherwise, returns #HADDR_UNDEF,
+ * \return Returns the offset in bytes; otherwise, returns HADDR_UNDEF,
  *         a negative value.
  *
  * \details H5Dget_offset() returns the address in the file of
@@ -783,6 +957,125 @@ H5_DLL herr_t H5Dread(hid_t dset_id, hid_t mem_type_id, hid_t mem_space_id, hid_
 H5_DLL herr_t H5Dwrite(hid_t dset_id, hid_t mem_type_id, hid_t mem_space_id, hid_t file_space_id,
                        hid_t dxpl_id, const void *buf);
 
+/**
+ * --------------------------------------------------------------------------
+ * \ingroup H5D
+ *
+ * \brief Writes a raw data chunk from a buffer directly to a dataset in a file
+ *
+ * \dset_id
+ * \dxpl_id
+ * \param[in]  filters  Mask for identifying the filters in use
+ * \param[in]  offset   Logical position of the chunk’s first element in the
+ *                      dataspace
+ * \param[in]  data_size    Size of the actual data to be written in bytes
+ * \param[in]  buf          Buffer containing data to be written to the chunk
+ *
+ * \return \herr_t
+ *
+ * \details H5Dwrite_chunk() writes a raw data chunk as specified
+ *          by its logical offset \p offset in a chunked dataset \p dset_id
+ *          from the application memory buffer \p buf to the dataset in
+ *          the file. Typically, the data in \p buf is preprocessed in
+ *          memory by a custom transformation, such as compression. The
+ *          chunk will bypass the library’s internal data transfer
+ *          pipeline, including filters, and will be written directly to
+ *          the file. Only one chunk can be written with this function.
+ *
+ *          \p filters is a mask providing a record of which filters are
+ *          used with the chunk. The default value of the mask is
+ *          zero (0), indicating that all enabled filters are applied. A
+ *          filter is skipped if the bit corresponding to the filter’s
+ *          position in the pipeline (0 ≤ position < 32) is turned on.
+ *          This mask is saved with the chunk in the file.
+ *
+ *          \p offset is an array specifying the logical position of the
+ *          first element of the chunk in the dataset’s dataspace. The
+ *          length of the offset array must equal the number of dimensions,
+ *          or rank, of the dataspace. The values in offset must not exceed
+ *          the dimension limits and must specify a point that falls on
+ *          a dataset chunk boundary.
+ *
+ *          \p data_size is the size in bytes of the chunk, representing
+ *          the number of bytes to be read from the buffer \p buf. If the
+ *          data chunk has been precompressed, \p data_size should be the
+ *          size of the compressed data.
+ *
+ *          \p buf is the memory buffer containing data to be written to
+ *          the chunk in the file.
+ *
+ * \attention Exercise caution when using H5Dread_chunk() and
+ *          H5Dwrite_chunk(), as they read and write data chunks directly
+ *          in a file. H5Dwrite_chunk() bypasses hyperslab selection, the
+ *          conversion of data from one datatype to another, and the filter
+ *          pipeline to write the chunk. Developers should have experience
+ *          with these processes before using this function.
+ *
+ * \note    H5Dread_chunk() and H5Dwrite_chunk() are currently not supported
+ *          with parallel HDF5 and do not support variable-length types.
+ *
+ * \since 1.10.2
+ *
+ */
+H5_DLL herr_t H5Dwrite_chunk(hid_t dset_id, hid_t dxpl_id, uint32_t filters, const hsize_t *offset,
+                             size_t data_size, const void *buf);
+
+/**
+ * --------------------------------------------------------------------------
+ * \ingroup H5D
+ *
+ * \brief Reads a raw data chunk directly from a dataset in a file into
+ * a buffer
+ *
+ * \dset_id
+ * \dxpl_id
+ * \param[in]  offset   Logical position of the chunk’s first element in the
+ *                      dataspace
+ * \param[in,out]  filters  Mask for identifying the filters in use
+ * \param[out]  buf     Buffer containing data to be written to the chunk
+ *
+ * \return \herr_t
+ *
+ * \details H5Dread_chunk() reads a raw data chunk as specified by
+ *          its logical offset \p offset in a chunked dataset \p dset_id
+ *          from the dataset in the file into the application memory
+ *          buffer \p buf. The data in \p buf is read directly from the
+ *          file bypassing the library’s internal data transfer pipeline,
+ *          including filters.
+ *
+ *          \p offset is an array specifying the logical position of the
+ *          first element of the chunk in the dataset’s dataspace. The
+ *          length of the \p offset array must equal the number of dimensions,
+ *          or rank, of the dataspace. The values in \p offset must not exceed
+ *          the dimension limits and must specify a point that falls on
+ *          a dataset chunk boundary.
+ *
+ *          The mask \p filters indicates which filters were used when the
+ *          chunk was written. A zero value (all bits 0) indicates that all
+ *          enabled filters are applied on the chunk. A filter is skipped if
+ *          the bit corresponding to the filter’s position in the pipeline
+ *          (0 ≤ position < 32) is turned on.
+ *
+ *          \p buf is the memory buffer containing the chunk read from
+ *          the dataset in the file.
+ *
+ * \attention Exercise caution when using H5Dread_chunk() and
+ *          H5Dwrite_chunk(), as they read and write data chunks directly
+ *          in a file. H5Dwrite_chunk() bypasses hyperslab selection, the
+ *          conversion of data from one datatype to another, and the filter
+ *          pipeline to write the chunk. Developers should have experience
+ *          with these processes before using this function. Please see
+ *          Using the Direct Chunk Write Function for more information.
+ *
+ * \note H5Dread_chunk() and H5Dwrite_chunk() are currently not supported
+ *       with parallel HDF5 and do not support variable-length datatypes.
+ *
+ * \since 1.10.2
+ *
+ */
+H5_DLL herr_t H5Dread_chunk(hid_t dset_id, hid_t dxpl_id, const hsize_t *offset, uint32_t *filters,
+                            void *buf);
+
 /**
  * --------------------------------------------------------------------------
  * \ingroup H5D
@@ -809,6 +1102,8 @@ H5_DLL herr_t H5Dwrite(hid_t dset_id, hid_t mem_type_id, hid_t mem_space_id, hid
  *            be restarted at the point of exit; a second H5Diterate()
  *            call will always restart at the beginning.
  *
+ * \warning   Modifying the selection of \p space_id during iteration
+ *            will lead to undefined behavior.
  *
  * \since 1.10.2
  *
@@ -977,6 +1272,55 @@ H5_DLL herr_t H5Dfill(const void *fill, hid_t fill_type_id, void *buf, hid_t buf
  */
 H5_DLL herr_t H5Dset_extent(hid_t dset_id, const hsize_t size[]);
 
+/**
+ * --------------------------------------------------------------------------
+ * \ingroup H5D
+ *
+ * \brief Flushes all buffers associated with a dataset to disk
+ *
+ * \dset_id
+ *
+ * \return \herr_t
+ *
+ * \details H5Dflush() causes all buffers associated with a
+ *          dataset to be immediately flushed to disk without removing
+ *          the data from the cache.
+ *
+ *          \note HDF5 does not possess full control over buffering.
+ *          H5Dflush() flushes the internal HDF5 buffers and then asks the
+ *          operating system (the OS) to flush the system buffers for the
+ *          open files. After that, the OS is responsible for ensuring
+ *          that the data is actually flushed to disk.
+ *
+ * \since 1.10.0
+ *
+ */
+H5_DLL herr_t H5Dflush(hid_t dset_id);
+
+/**
+ * --------------------------------------------------------------------------
+ * \ingroup H5D
+ *
+ * \brief Refreshes all buffers associated with a dataset
+ *
+ * \dset_id
+ *
+ * \return \herr_t
+ *
+ * \details H5Drefresh() causes all buffers associated with a
+ *          dataset to be cleared and immediately re-loaded with updated
+ *          contents from disk.
+ *
+ *          This function essentially closes the dataset, evicts all
+ *          metadata associated with it from the cache, and then re-opens
+ *          the dataset. The reopened dataset is automatically re-registered
+ *          with the same identifier.
+ *
+ * \since 1.10.2
+ *
+ */
+H5_DLL herr_t H5Drefresh(hid_t dset_id);
+
 /**
  * --------------------------------------------------------------------------
  * \ingroup H5D
@@ -996,7 +1340,7 @@ H5_DLL herr_t H5Dset_extent(hid_t dset_id, const hsize_t size[]);
  *          \p op and scatters it to the supplied buffer \p dst_buf in a
  *          manner similar to data being written to a dataset.
  *
- *          \p dst_space_id is a dataspace which defines the extent of \p
+ *          \p dst_space_id is a dataspace that defines the extent of \p
  *          dst_buf and the selection within it to scatter the data to.
  *
  *          \p type_id is the datatype of the data to be scattered in both
@@ -1053,7 +1397,7 @@ H5_DLL herr_t H5Dscatter(H5D_scatter_func_t op, void *op_data, hid_t type_id, hi
  *          enough to hold all the data if the callback function \p op is
  *          not provided.
  *
- *          \p op is a callback function which handles the gathered data.
+ *          \p op is a callback function that handles the gathered data.
  *          It is optional if \p dst_buf is large enough to hold all of the
  *          gathered data; required otherwise.
  *
@@ -1099,8 +1443,12 @@ H5_DLL herr_t H5Dgather(hid_t src_space_id, const void *src_buf, hid_t type_id,
  */
 H5_DLL herr_t H5Dclose(hid_t dset_id);
 
+/// \cond DEV
 /* Internal API routines */
 H5_DLL herr_t H5Ddebug(hid_t dset_id);
+H5_DLL herr_t H5Dformat_convert(hid_t dset_id);
+H5_DLL herr_t H5Dget_chunk_index_type(hid_t did, H5D_chunk_index_t *idx_type);
+/// \endcond
 
 /* Symbols defined for compatibility with previous versions of the HDF5 API.
  *
@@ -1109,6 +1457,20 @@ H5_DLL herr_t H5Ddebug(hid_t dset_id);
 #ifndef H5_NO_DEPRECATED_SYMBOLS
 
 /* Macros */
+#define H5D_CHUNK_BTREE H5D_CHUNK_IDX_BTREE
+
+/* Formerly used to support the H5DOread/write_chunk() API calls.
+ * These symbols are no longer used in the library.
+ */
+/* Property names for H5DOwrite_chunk */
+#define H5D_XFER_DIRECT_CHUNK_WRITE_FLAG_NAME     "direct_chunk_flag"
+#define H5D_XFER_DIRECT_CHUNK_WRITE_FILTERS_NAME  "direct_chunk_filters"
+#define H5D_XFER_DIRECT_CHUNK_WRITE_OFFSET_NAME   "direct_chunk_offset"
+#define H5D_XFER_DIRECT_CHUNK_WRITE_DATASIZE_NAME "direct_chunk_datasize"
+/* Property names for H5DOread_chunk */
+#define H5D_XFER_DIRECT_CHUNK_READ_FLAG_NAME    "direct_chunk_read_flag"
+#define H5D_XFER_DIRECT_CHUNK_READ_OFFSET_NAME  "direct_chunk_read_offset"
+#define H5D_XFER_DIRECT_CHUNK_READ_FILTERS_NAME "direct_chunk_read_filters"
 
 /* Typedefs */
 
@@ -1165,10 +1527,10 @@ H5_DLL herr_t H5Ddebug(hid_t dset_id);
  *
  *          H5Dcreate() and H5Dcreate_anon() return a dataset identifier for
  *          success or a negative value for failure. The dataset identifier
- *          should eventually be closed by calling H5Dclose() to release
+ *          should eventually be closed by calling H5Dclose() to release the
  *          resources it uses.
  *
- *          See H5Dcreate_anon() for discussion of the differences between
+ *          See H5Dcreate_anon() for a discussion of the differences between
  *          H5Dcreate() and H5Dcreate_anon().
  *
  *          The HDF5 library provides flexible means of specifying a fill value,
@@ -1244,7 +1606,7 @@ H5_DLL hid_t H5Dopen1(hid_t loc_id, const char *name);
  *
  *          This function ensures that the dataset dimensions are of at least
  *          the sizes specified in size. The function H5Dset_extent() must be
- *          used if the dataset dimension sizes are are to be reduced.
+ *          used if the dataset dimension sizes are to be reduced.
  *
  * \version 1.8.0 Function deprecated in this release. Parameter size
  *                syntax changed to \Code{const hsize_t size[]} in this release.
