@@ -1,6 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright by The HDF Group.                                               *
- * Copyright by the Board of Trustees of the University of Illinois.         *
  * All rights reserved.                                                      *
  *                                                                           *
  * This file is part of HDF5.  The full HDF5 copyright notice, including     *
@@ -18,8 +17,8 @@
  * Purpose:     Data filter pipeline message.
  */
 
-#define H5O_PACKAGE /*suppress error about including H5Opkg      */
-#define H5Z_PACKAGE /*suppress error about including H5Zpkg      */
+#include "H5Omodule.h" /* This source code file is part of the H5O module */
+#define H5Z_FRIEND     /*suppress error about including H5Zpkg      */
 
 #include "H5private.h"   /* Generic Functions            */
 #include "H5Dprivate.h"  /* Datasets                     */
@@ -31,36 +30,35 @@
 
 /* PRIVATE PROTOTYPES */
 static herr_t H5O_pline_encode(H5F_t *f, uint8_t *p, const void *mesg);
-static void *H5O_pline_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh, unsigned mesg_flags, unsigned *ioflags,
-                              size_t p_size, const uint8_t *p);
-static void *H5O_pline_copy(const void *_mesg, void *_dest);
+static void  *H5O__pline_decode(H5F_t *f, H5O_t *open_oh, unsigned mesg_flags, unsigned *ioflags,
+                                size_t p_size, const uint8_t *p);
+static void  *H5O_pline_copy(const void *_mesg, void *_dest);
 static size_t H5O_pline_size(const H5F_t *f, const void *_mesg);
-static herr_t H5O_pline_reset(void *_mesg);
-static herr_t H5O_pline_free(void *_mesg);
+static herr_t H5O__pline_reset(void *_mesg);
+static herr_t H5O__pline_free(void *_mesg);
 static herr_t H5O_pline_pre_copy_file(H5F_t *file_src, const void *mesg_src, hbool_t *deleted,
                                       const H5O_copy_t *cpy_info, void *_udata);
-static herr_t H5O_pline_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg, FILE *stream, int indent,
-                              int fwidth);
+static herr_t H5O__pline_debug(H5F_t *f, const void *_mesg, FILE *stream, int indent, int fwidth);
 
 /* Set up & include shared message "interface" info */
 #define H5O_SHARED_TYPE        H5O_MSG_PLINE
 #define H5O_SHARED_DECODE      H5O_pline_shared_decode
-#define H5O_SHARED_DECODE_REAL H5O_pline_decode
+#define H5O_SHARED_DECODE_REAL H5O__pline_decode
 #define H5O_SHARED_ENCODE      H5O_pline_shared_encode
 #define H5O_SHARED_ENCODE_REAL H5O_pline_encode
 #define H5O_SHARED_SIZE        H5O_pline_shared_size
 #define H5O_SHARED_SIZE_REAL   H5O_pline_size
-#define H5O_SHARED_DELETE      H5O_pline_shared_delete
+#define H5O_SHARED_DELETE      H5O__pline_shared_delete
 #undef H5O_SHARED_DELETE_REAL
-#define H5O_SHARED_LINK H5O_pline_shared_link
+#define H5O_SHARED_LINK H5O__pline_shared_link
 #undef H5O_SHARED_LINK_REAL
-#define H5O_SHARED_COPY_FILE H5O_pline_shared_copy_file
+#define H5O_SHARED_COPY_FILE H5O__pline_shared_copy_file
 #undef H5O_SHARED_COPY_FILE_REAL
 #define H5O_SHARED_POST_COPY_FILE H5O_pline_shared_post_copy_file
 #undef H5O_SHARED_POST_COPY_FILE_REAL
 #undef H5O_SHARED_POST_COPY_FILE_UPD
 #define H5O_SHARED_DEBUG      H5O_pline_shared_debug
-#define H5O_SHARED_DEBUG_REAL H5O_pline_debug
+#define H5O_SHARED_DEBUG_REAL H5O__pline_debug
 #include "H5Oshared.h" /* Shared Object Header Message Callbacks */
 
 /* This message derives from H5O message class */
@@ -68,30 +66,37 @@ const H5O_msg_class_t H5O_MSG_PLINE[1] = {{
     H5O_PLINE_ID,                              /* message id number        */
     "filter pipeline",                         /* message name for debugging    */
     sizeof(H5O_pline_t),                       /* native message size        */
-    H5O_SHARE_IS_SHARABLE | H5O_SHARE_IN_OHDR, /* messages are sharable?       */
+    H5O_SHARE_IS_SHARABLE | H5O_SHARE_IN_OHDR, /* messages are shareable?       */
     H5O_pline_shared_decode,                   /* decode message        */
     H5O_pline_shared_encode,                   /* encode message        */
     H5O_pline_copy,                            /* copy the native value    */
     H5O_pline_shared_size,                     /* size of raw message        */
-    H5O_pline_reset,                           /* reset method            */
-    H5O_pline_free,                            /* free method            */
-    H5O_pline_shared_delete,                   /* file delete method        */
-    H5O_pline_shared_link,                     /* link method            */
+    H5O__pline_reset,                          /* reset method            */
+    H5O__pline_free,                           /* free method            */
+    H5O__pline_shared_delete,                  /* file delete method        */
+    H5O__pline_shared_link,                    /* link method            */
     NULL,                                      /* set share method        */
     NULL,                                      /*can share method        */
     H5O_pline_pre_copy_file,                   /* pre copy native value to file */
-    H5O_pline_shared_copy_file,                /* copy native value to file    */
+    H5O__pline_shared_copy_file,               /* copy native value to file    */
     H5O_pline_shared_post_copy_file,           /* post copy native value to file    */
     NULL,                                      /* get creation index        */
     NULL,                                      /* set creation index        */
     H5O_pline_shared_debug                     /* debug the message        */
 }};
 
+/* Format version bounds for filter pipeline */
+const unsigned H5O_pline_ver_bounds[] = {
+    H5O_PLINE_VERSION_1,     /* H5F_LIBVER_EARLIEST */
+    H5O_PLINE_VERSION_2,     /* H5F_LIBVER_V18 */
+    H5O_PLINE_VERSION_LATEST /* H5F_LIBVER_LATEST */
+};
+
 /* Declare a free list to manage the H5O_pline_t struct */
 H5FL_DEFINE(H5O_pline_t);
 
 /*-------------------------------------------------------------------------
- * Function:    H5O_pline_decode
+ * Function:    H5O__pline_decode
  *
  * Purpose:     Decodes a filter pipeline message.
  *
@@ -103,10 +108,10 @@ H5FL_DEFINE(H5O_pline_t);
  *
  *-------------------------------------------------------------------------
  */
+
 static void *
-H5O_pline_decode(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, H5O_t H5_ATTR_UNUSED *open_oh,
-                 unsigned H5_ATTR_UNUSED mesg_flags, unsigned H5_ATTR_UNUSED *ioflags, size_t p_size,
-                 const uint8_t *p)
+H5O__pline_decode(H5F_t H5_ATTR_UNUSED *f, H5O_t H5_ATTR_UNUSED *open_oh, unsigned H5_ATTR_UNUSED mesg_flags,
+                  unsigned H5_ATTR_UNUSED *ioflags, size_t p_size, const uint8_t *p)
 {
     H5O_pline_t       *pline = NULL;               /* Pipeline message */
     H5Z_filter_info_t *filter;                     /* Filter to decode */
@@ -115,7 +120,7 @@ H5O_pline_decode(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, H5O_t H5
     const uint8_t     *p_end     = p + p_size - 1; /* End of the p buffer */
     void              *ret_value = NULL;           /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT
+    FUNC_ENTER_STATIC
 
     /* check args */
     HDassert(p);
@@ -125,6 +130,9 @@ H5O_pline_decode(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, H5O_t H5
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
     /* Version */
+    if (p + 4 - 1 > p_end) /* 4 byte is minimum for all versions */
+        HGOTO_ERROR(H5E_OHDR, H5E_NOSPACE, NULL, "ran off the end of the buffer: current p = %p, p_end = %p",
+                    p + 4, p_end)
     pline->version = *p++;
     if (pline->version < H5O_PLINE_VERSION_1 || pline->version > H5O_PLINE_VERSION_LATEST)
         HGOTO_ERROR(H5E_PLINE, H5E_CANTLOAD, NULL, "bad version number for filter pipeline message")
@@ -153,6 +161,9 @@ H5O_pline_decode(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, H5O_t H5
     /* Decode filters */
     for (i = 0, filter = &pline->filter[0]; i < pline->nused; i++, filter++) {
         /* Filter ID */
+        if (p + 6 - 1 > p_end) /* 6 bytes minimum */
+            HGOTO_ERROR(H5E_OHDR, H5E_NOSPACE, NULL,
+                        "ran off the end of the buffer: current p = %p, p_end = %p", p + 6, p_end)
         UINT16DECODE(p, filter->id);
 
         /* Length of filter name */
@@ -162,6 +173,9 @@ H5O_pline_decode(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, H5O_t H5
             UINT16DECODE(p, name_length);
             if (pline->version == H5O_PLINE_VERSION_1 && name_length % 8)
                 HGOTO_ERROR(H5E_PLINE, H5E_CANTLOAD, NULL, "filter name length is not a multiple of eight")
+            if (p + 4 - 1 > p_end) /* with name_length 4 bytes to go */
+                HGOTO_ERROR(H5E_OHDR, H5E_NOSPACE, NULL,
+                            "ran off the end of the buffer: current p = %p, p_end = %p", p + 4, p_end)
         } /* end if */
 
         /* Filter flags */
@@ -173,9 +187,12 @@ H5O_pline_decode(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, H5O_t H5
         /* Filter name, if there is one */
         if (name_length) {
             size_t actual_name_length; /* Actual length of name */
-
+            size_t len = (size_t)(p_end - p + 1);
             /* Determine actual name length (without padding, but with null terminator) */
-            actual_name_length = HDstrlen((const char *)p) + 1;
+            actual_name_length = HDstrnlen((const char *)p, len);
+            if (actual_name_length == len)
+                HGOTO_ERROR(H5E_OHDR, H5E_NOSPACE, NULL, "filter name not null terminated")
+            actual_name_length += 1; /* include \0 byte */
             HDassert(actual_name_length <= name_length);
 
             /* Allocate space for the filter name, or use the internal buffer */
@@ -227,12 +244,12 @@ H5O_pline_decode(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, H5O_t H5
 
 done:
     if (NULL == ret_value && pline) {
-        H5O_pline_reset(pline);
-        H5O_pline_free(pline);
+        H5O__pline_reset(pline);
+        H5O__pline_free(pline);
     } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5O_pline_decode() */
+} /* end H5O__pline_decode() */
 
 /*-------------------------------------------------------------------------
  * Function:    H5O_pline_encode
@@ -309,7 +326,7 @@ H5O_pline_encode(H5F_t H5_ATTR_UNUSED *f, uint8_t *p /*out*/, const void *mesg)
         /* Encode name, if there is one to encode */
         if (name_length > 0) {
             /* Store name, with null terminator */
-            HDmemcpy(p, name, name_length);
+            H5MM_memcpy(p, name, name_length);
             p += name_length;
 
             /* Pad out name to alignment, in older versions */
@@ -401,8 +418,8 @@ H5O_pline_copy(const void *_src, void *_dst /*out*/)
                                      (unsigned *)H5MM_malloc(src->filter[i].cd_nelmts * sizeof(unsigned))))
                         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
-                    HDmemcpy(dst->filter[i].cd_values, src->filter[i].cd_values,
-                             src->filter[i].cd_nelmts * sizeof(unsigned));
+                    H5MM_memcpy(dst->filter[i].cd_values, src->filter[i].cd_values,
+                                src->filter[i].cd_nelmts * sizeof(unsigned));
                 } /* end if */
                 else
                     dst->filter[i].cd_values = dst->filter[i]._cd_values;
@@ -417,9 +434,9 @@ H5O_pline_copy(const void *_src, void *_dst /*out*/)
 
 done:
     if (!ret_value && dst) {
-        H5O_pline_reset(dst);
+        H5O__pline_reset(dst);
         if (!_dst)
-            H5O_pline_free(dst);
+            H5O__pline_free(dst);
     } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -449,9 +466,9 @@ H5O_pline_size(const H5F_t H5_ATTR_UNUSED *f, const void *mesg)
     FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Message header */
-    ret_value = 1 +                                              /*version            */
-                1 +                                              /*number of filters        */
-                (pline->version == H5O_PLINE_VERSION_1 ? 6 : 0); /*reserved            */
+    ret_value = (size_t)(1 +                                               /*version            */
+                         1 +                                               /*number of filters        */
+                         (pline->version == H5O_PLINE_VERSION_1 ? 6 : 0)); /*reserved            */
 
     /* Calculate size of each filter in pipeline */
     for (i = 0; i < pline->nused; i++) {
@@ -490,7 +507,7 @@ H5O_pline_size(const H5F_t H5_ATTR_UNUSED *f, const void *mesg)
 } /* end H5O_pline_size() */
 
 /*-------------------------------------------------------------------------
- * Function:    H5O_pline_reset
+ * Function:    H5O__pline_reset
  *
  * Purpose:    Resets a filter pipeline message by clearing all filters.
  *        The MESG buffer is not freed.
@@ -503,12 +520,12 @@ H5O_pline_size(const H5F_t H5_ATTR_UNUSED *f, const void *mesg)
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5O_pline_reset(void *mesg)
+H5O__pline_reset(void *mesg)
 {
     H5O_pline_t *pline = (H5O_pline_t *)mesg; /* Pipeline message */
     size_t       i;                           /* Local index variable */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
+    FUNC_ENTER_STATIC_NOERR
 
     /* NOTE: This function can be called during error processing from
      *       other API calls so DO NOT ASSUME THAT ANY VALUES ARE SANE.
@@ -541,10 +558,10 @@ H5O_pline_reset(void *mesg)
     pline->version = H5O_PLINE_VERSION_1;
 
     FUNC_LEAVE_NOAPI(SUCCEED)
-} /* end H5O_pline_reset() */
+} /* end H5O__pline_reset() */
 
 /*-------------------------------------------------------------------------
- * Function:    H5O_pline_free
+ * Function:    H5O__pline_free
  *
  * Purpose:    Frees the message
  *
@@ -556,16 +573,16 @@ H5O_pline_reset(void *mesg)
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5O_pline_free(void *mesg)
+H5O__pline_free(void *mesg)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
+    FUNC_ENTER_STATIC_NOERR
 
     HDassert(mesg);
 
     mesg = H5FL_FREE(H5O_pline_t, mesg);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
-} /* end H5O_pline_free() */
+} /* end H5O__pline_free() */
 
 /*-------------------------------------------------------------------------
  * Function:    H5O_pline_pre_copy_file
@@ -584,9 +601,9 @@ H5O_pline_free(void *mesg)
  */
 static herr_t
 H5O_pline_pre_copy_file(H5F_t H5_ATTR_UNUSED *file_src, const void *mesg_src, hbool_t H5_ATTR_UNUSED *deleted,
-                        const H5O_copy_t H5_ATTR_UNUSED *cpy_info, void *_udata)
+                        const H5O_copy_t *cpy_info, void *_udata)
 {
-    const H5O_pline_t         *pline_src = (const H5O_pline_t *)mesg_src;       /* Source datatype */
+    const H5O_pline_t         *pline_src = (const H5O_pline_t *)mesg_src;       /* Source pline */
     H5O_copy_file_ud_common_t *udata     = (H5O_copy_file_ud_common_t *)_udata; /* Object copying user data */
     herr_t                     ret_value = SUCCEED;                             /* Return value */
 
@@ -594,6 +611,13 @@ H5O_pline_pre_copy_file(H5F_t H5_ATTR_UNUSED *file_src, const void *mesg_src, hb
 
     /* check args */
     HDassert(pline_src);
+    HDassert(cpy_info);
+    HDassert(cpy_info->file_dst);
+
+    /* Check to ensure that the version of the message to be copied does not exceed
+       the message version allowed by the destination file's high bound */
+    if (pline_src->version > H5O_pline_ver_bounds[H5F_HIGH_BOUND(cpy_info->file_dst)])
+        HGOTO_ERROR(H5E_OHDR, H5E_BADRANGE, FAIL, "pline message version out of bounds")
 
     /* If the user data is non-NULL, assume we are copying a dataset or group
      * and make a copy of the filter pipeline for later in
@@ -608,7 +632,7 @@ done:
 } /* end H5O_pline_pre_copy_file() */
 
 /*-------------------------------------------------------------------------
- * Function:    H5O_pline_debug
+ * Function:    H5O__pline_debug
  *
  * Purpose:    Prints debugging information for filter pipeline message MESG
  *        on output stream STREAM.  Each line is indented INDENT
@@ -622,13 +646,12 @@ done:
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5O_pline_debug(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, const void *mesg, FILE *stream,
-                int indent, int fwidth)
+H5O__pline_debug(H5F_t H5_ATTR_UNUSED *f, const void *mesg, FILE *stream, int indent, int fwidth)
 {
     const H5O_pline_t *pline = (const H5O_pline_t *)mesg;
     size_t             i, j;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
+    FUNC_ENTER_STATIC_NOERR
 
     /* check args */
     HDassert(f);
@@ -637,14 +660,14 @@ H5O_pline_debug(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, const voi
     HDassert(indent >= 0);
     HDassert(fwidth >= 0);
 
-    HDfprintf(stream, "%*s%-*s %Zu/%Zu\n", indent, "", fwidth, "Number of filters:", pline->nused,
+    HDfprintf(stream, "%*s%-*s %zu/%zu\n", indent, "", fwidth, "Number of filters:", pline->nused,
               pline->nalloc);
 
     /* Loop over all the filters */
     for (i = 0; i < pline->nused; i++) {
         char name[32];
 
-        HDsnprintf(name, sizeof(name), "Filter at position %u", (unsigned)i);
+        HDsnprintf(name, sizeof(name), "Filter at position %zu", i);
         HDfprintf(stream, "%*s%-*s\n", indent, "", fwidth, name);
         HDfprintf(stream, "%*s%-*s 0x%04x\n", indent + 3, "", MAX(0, fwidth - 3),
                   "Filter identification:", (unsigned)(pline->filter[i].id));
@@ -655,7 +678,7 @@ H5O_pline_debug(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, const voi
             HDfprintf(stream, "%*s%-*s NONE\n", indent + 3, "", MAX(0, fwidth - 3), "Filter name:");
         HDfprintf(stream, "%*s%-*s 0x%04x\n", indent + 3, "", MAX(0, fwidth - 3),
                   "Flags:", pline->filter[i].flags);
-        HDfprintf(stream, "%*s%-*s %Zu\n", indent + 3, "", MAX(0, fwidth - 3),
+        HDfprintf(stream, "%*s%-*s %zu\n", indent + 3, "", MAX(0, fwidth - 3),
                   "Num CD values:", pline->filter[i].cd_nelmts);
 
         /* Filter parameters */
@@ -669,30 +692,41 @@ H5O_pline_debug(H5F_t H5_ATTR_UNUSED *f, hid_t H5_ATTR_UNUSED dxpl_id, const voi
     }     /* end for */
 
     FUNC_LEAVE_NOAPI(SUCCEED)
-} /* end H5O_pline_debug() */
+} /* end H5O__pline_debug() */
 
 /*-------------------------------------------------------------------------
- * Function:    H5O_pline_set_latest_version
+ * Function:    H5O_pline_set_version
  *
- * Purpose:     Set the encoding for a I/O filter pipeline to the latest version.
+ * Purpose:     Set the version to encode an I/O filter pipeline with.
  *
  * Return:      Non-negative on success/Negative on failure
  *
- * Programmer:  Quincey Koziol
- *              Tuesday, July 24, 2007
+ * Programmer:  Vailin Choi; December 2017
  *
  *-------------------------------------------------------------------------
  */
 herr_t
-H5O_pline_set_latest_version(H5O_pline_t *pline)
+H5O_pline_set_version(H5F_t *f, H5O_pline_t *pline)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
+    unsigned version;             /* Message version */
+    herr_t   ret_value = SUCCEED; /* Return value */
+
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Sanity check */
+    HDassert(f);
     HDassert(pline);
 
-    /* Set encoding of I/O pipeline to latest version */
-    pline->version = H5O_PLINE_VERSION_LATEST;
+    /* Upgrade to the version indicated by the file's low bound if higher */
+    version = MAX(pline->version, H5O_pline_ver_bounds[H5F_LOW_BOUND(f)]);
 
-    FUNC_LEAVE_NOAPI(SUCCEED)
-} /* end H5O_pline_set_latest_version() */
+    /* Version bounds check */
+    if (version > H5O_pline_ver_bounds[H5F_HIGH_BOUND(f)])
+        HGOTO_ERROR(H5E_PLINE, H5E_BADRANGE, FAIL, "Filter pipeline version out of bounds")
+
+    /* Set the message version */
+    pline->version = version;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O_pline_set_version() */
