@@ -1,6 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright by The HDF Group.                                               *
- * Copyright by the Board of Trustees of the University of Illinois.         *
  * All rights reserved.                                                      *
  *                                                                           *
  * This file is part of HDF5.  The full HDF5 copyright notice, including     *
@@ -15,17 +14,15 @@
 /* Module Setup */
 /****************/
 
-#define H5S_PACKAGE /*suppress error about including H5Spkg	  */
-
-/* Interface initialization */
-#define H5_INTERFACE_INIT_FUNC H5S_init_interface
+#include "H5Smodule.h" /* This source code file is part of the H5S module */
 
 /***********/
 /* Headers */
 /***********/
-#include "H5private.h"   /* Generic Functions			*/
-#include "H5Eprivate.h"  /* Error handling		  	*/
-#include "H5Fprivate.h"  /* Files				*/
+#include "H5private.h"   /* Generic Functions            */
+#include "H5Eprivate.h"  /* Error handling              */
+#include "H5CXprivate.h" /* API Contexts         */
+#include "H5Fprivate.h"  /* Files                */
 #include "H5FLprivate.h" /* Free lists                           */
 #include "H5Iprivate.h"  /* IDs                      */
 #include "H5MMprivate.h" /* Memory management            */
@@ -46,15 +43,26 @@
 /********************/
 /* Local Prototypes */
 /********************/
-static herr_t H5S_set_extent_simple(H5S_t *space, unsigned rank, const hsize_t *dims, const hsize_t *max);
-static htri_t H5S_is_simple(const H5S_t *sdim);
-static herr_t H5S_encode(H5S_t *obj, unsigned char *buf, size_t *nalloc);
-static H5S_t *H5S_decode(const unsigned char *buf);
+static htri_t H5S__is_simple(const H5S_t *sdim);
+
+/*****************************/
+/* Library Private Variables */
+/*****************************/
 
 /*********************/
 /* Package Variables */
 /*********************/
 
+/* Package initialization variable */
+hbool_t H5_PKG_INIT_VAR = FALSE;
+
+/* Format version bounds for dataspace */
+const unsigned H5O_sdspace_ver_bounds[] = {
+    H5O_SDSPACE_VERSION_1,     /* H5F_LIBVER_EARLIEST */
+    H5O_SDSPACE_VERSION_2,     /* H5F_LIBVER_V18 */
+    H5O_SDSPACE_VERSION_LATEST /* H5F_LIBVER_LATEST */
+};
+
 /* Declare a free list to manage the H5S_extent_t struct */
 H5FL_DEFINE(H5S_extent_t);
 
@@ -71,48 +79,54 @@ H5FL_ARR_DEFINE(hsize_t, H5S_MAX_RANK);
 /* Dataspace ID class */
 static const H5I_class_t H5I_DATASPACE_CLS[1] = {{
     H5I_DATASPACE,        /* ID class value */
-    H5I_CLASS_REUSE_IDS,  /* Class flags */
+    0,                    /* Class flags */
     2,                    /* # of reserved IDs for class */
     (H5I_free_t)H5S_close /* Callback routine for closing objects of this class */
 }};
 
+/* Flag indicating "top" of interface has been initialized */
+static hbool_t H5S_top_package_initialize_s = FALSE;
+
 /*--------------------------------------------------------------------------
 NAME
-   H5S_init_interface -- Initialize interface-specific information
+   H5S__init_package -- Initialize interface-specific information
 USAGE
-    herr_t H5S_init_interface()
-
+    herr_t H5S__init_package()
 RETURNS
     Non-negative on success/Negative on failure
 DESCRIPTION
     Initializes any interface-specific data or routines.
 --------------------------------------------------------------------------*/
-static herr_t
-H5S_init_interface(void)
+herr_t
+H5S__init_package(void)
 {
     herr_t ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT
+    FUNC_ENTER_PACKAGE
 
     /* Initialize the atom group for the dataspace IDs */
     if (H5I_register_type(H5I_DATASPACE_CLS) < 0)
-        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to initialize interface")
+        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to initialize dataspace ID class")
+
+    /* Mark "top" of interface as initialized, too */
+    H5S_top_package_initialize_s = TRUE;
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5S_init_interface() */
+} /* end H5S__init_package() */
 
 /*--------------------------------------------------------------------------
  NAME
-    H5S_term_interface
+    H5S_top_term_package
  PURPOSE
     Terminate various H5S objects
  USAGE
-    void H5S_term_interface()
+    void H5S_top_term_package()
  RETURNS
     Non-negative on success/Negative on failure
  DESCRIPTION
-    Release the atom group and any other resources allocated.
+    Release IDs for the atom group, deferring full interface shutdown
+    until later (in H5S_term_package).
  GLOBAL VARIABLES
  COMMENTS, BUGS, ASSUMPTIONS
      Can't report errors...
@@ -120,29 +134,117 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 int
-H5S_term_interface(void)
+H5S_top_term_package(void)
 {
     int n = 0;
 
     FUNC_ENTER_NOAPI_NOINIT_NOERR
 
-    if (H5_interface_initialize_g) {
+    if (H5S_top_package_initialize_s) {
         if (H5I_nmembers(H5I_DATASPACE) > 0) {
             (void)H5I_clear_type(H5I_DATASPACE, FALSE, FALSE);
             n++; /*H5I*/
         }        /* end if */
-        else {
-            /* Destroy the dataspace object id group */
-            (void)H5I_dec_type_ref(H5I_DATASPACE);
-            n++; /*H5I*/
 
-            /* Shut down interface */
-            H5_interface_initialize_g = 0;
-        } /* end else */
-    }     /* end if */
+        /* Mark "top" of interface as closed */
+        if (0 == n)
+            H5S_top_package_initialize_s = FALSE;
+    } /* end if */
 
     FUNC_LEAVE_NOAPI(n)
-} /* end H5S_term_interface() */
+} /* end H5S_top_term_package() */
+
+/*--------------------------------------------------------------------------
+ NAME
+    H5S_term_package
+ PURPOSE
+    Terminate various H5S objects
+ USAGE
+    void H5S_term_package()
+ RETURNS
+    Non-negative on success/Negative on failure
+ DESCRIPTION
+    Release the atom group and any other resources allocated.
+ GLOBAL VARIABLES
+ COMMENTS, BUGS, ASSUMPTIONS
+     Can't report errors...
+
+     Finishes shutting down the interface, after H5S_top_term_package()
+     is called
+ EXAMPLES
+ REVISION LOG
+--------------------------------------------------------------------------*/
+int
+H5S_term_package(void)
+{
+    int n = 0;
+
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
+
+    if (H5_PKG_INIT_VAR) {
+        /* Sanity checks */
+        HDassert(0 == H5I_nmembers(H5I_DATASPACE));
+        HDassert(FALSE == H5S_top_package_initialize_s);
+
+        /* Destroy the dataspace object id group */
+        n += (H5I_dec_type_ref(H5I_DATASPACE) > 0);
+
+        /* Mark interface as closed */
+        if (0 == n)
+            H5_PKG_INIT_VAR = FALSE;
+    } /* end if */
+
+    FUNC_LEAVE_NOAPI(n)
+} /* end H5S_term_package() */
+
+/*--------------------------------------------------------------------------
+ NAME
+    H5S_get_validiated_dataspace
+ PURPOSE
+    Get a pointer to a validated H5S_t pointer
+ USAGE
+   H5S_t *H5S_get_validated_space(dataspace_id, space)
+    hid_t           space_id;       IN: The ID of the dataspace
+    const H5S_t *   space;          OUT: A pointer to the dataspace
+ RETURNS
+    SUCCEED/FAIL
+ DESCRIPTION
+    Gets a pointer to a dataspace struct after validating it. The pointer
+    can be NULL (if the ID is H5S_ALL, for example).
+ GLOBAL VARIABLES
+ COMMENTS, BUGS, ASSUMPTIONS
+ EXAMPLES
+ REVISION LOG
+--------------------------------------------------------------------------*/
+herr_t
+H5S_get_validated_dataspace(hid_t space_id, H5S_t **space)
+{
+    herr_t ret_value = SUCCEED; /* Return value */
+
+    FUNC_ENTER_NOAPI(FAIL)
+
+    HDassert(space);
+
+    /* Check for invalid ID */
+    if (space_id < 0)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "invalid space_id (ID cannot be a negative number)")
+
+    /* No special dataspace struct for H5S_ALL */
+    if (H5S_ALL == space_id)
+        *space = NULL;
+    else {
+        /* Get the dataspace pointer */
+        if (NULL == (*space = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
+            HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "space_id is not a dataspace ID")
+
+        /* Check for valid selection */
+        if (H5S_SELECT_VALID(*space) != TRUE)
+            HGOTO_ERROR(H5E_DATASPACE, H5E_BADRANGE, FAIL, "selection + offset not within extent")
+    }
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5S_get_validated_dataspace() */
 
 /*--------------------------------------------------------------------------
  NAME
@@ -268,7 +370,7 @@ done:
 } /* end H5Screate() */
 
 /*-------------------------------------------------------------------------
- * Function:    H5S_extent_release
+ * Function:    H5S__extent_release
  *
  * Purpose:     Releases all memory associated with a dataspace extent.
  *
@@ -280,11 +382,9 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5S_extent_release(H5S_extent_t *extent)
+H5S__extent_release(H5S_extent_t *extent)
 {
-    herr_t ret_value = SUCCEED; /* Return value */
-
-    FUNC_ENTER_NOAPI(FAIL)
+    FUNC_ENTER_PACKAGE_NOERR
 
     HDassert(extent);
 
@@ -296,9 +396,11 @@ H5S_extent_release(H5S_extent_t *extent)
             extent->max = H5FL_ARR_FREE(hsize_t, extent->max);
     } /* end if */
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5S_extent_release() */
+    extent->rank  = 0;
+    extent->nelem = 0;
+
+    FUNC_LEAVE_NOAPI(SUCCEED)
+} /* end H5S__extent_release() */
 
 /*-------------------------------------------------------------------------
  * Function:    H5S_close
@@ -323,16 +425,20 @@ H5S_close(H5S_t *ds)
 
     /* Release selection (this should come before the extent release) */
     if (H5S_SELECT_RELEASE(ds) < 0)
-        HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace selection")
+        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace selection")
 
     /* Release extent */
-    if (H5S_extent_release(&ds->extent) < 0)
-        HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace extent")
-
-    /* Release the main structure */
-    ds = H5FL_FREE(H5S_t, ds);
+    if (H5S__extent_release(&ds->extent) < 0)
+        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace extent")
 
 done:
+    /* Release the main structure.
+     * Always do this to ensure that we don't leak memory when calling this
+     * function on partially constructed dataspaces (which will fail one or
+     * both of the above calls)
+     */
+    H5FL_FREE(H5S_t, ds);
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5S_close() */
 
@@ -405,7 +511,7 @@ H5Scopy(hid_t space_id)
         HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, H5I_INVALID_HID, "unable to register dataspace atom")
 
 done:
-    if (H5I_INVALID_HID == ret_value)
+    if (ret_value < 0)
         if (dst && H5S_close(dst) < 0)
             HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, H5I_INVALID_HID, "unable to release dataspace")
 
@@ -441,15 +547,9 @@ H5Sextent_copy(hid_t dst_id, hid_t src_id)
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
     /* Copy */
-    if (H5S_extent_copy(&(dst->extent), &(src->extent), TRUE) < 0)
+    if (H5S_extent_copy(dst, src) < 0)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCOPY, FAIL, "can't copy extent")
 
-    /* If the selection is 'all', update the number of elements selected in the
-     * destination space */
-    if (H5S_SEL_ALL == H5S_GET_SELECT_TYPE(dst))
-        if (H5S_select_all(dst, FALSE) < 0)
-            HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDELETE, FAIL, "can't change selection")
-
 done:
     FUNC_LEAVE_API(ret_value)
 } /* end H5Sextent_copy() */
@@ -467,15 +567,51 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5S_extent_copy(H5S_extent_t *dst, const H5S_extent_t *src, hbool_t copy_max)
+H5S_extent_copy(H5S_t *dst, const H5S_t *src)
+{
+    herr_t ret_value = SUCCEED; /* Return value */
+
+    FUNC_ENTER_NOAPI(FAIL)
+
+    HDassert(dst);
+    HDassert(src);
+
+    /* Copy extent */
+    if (H5S__extent_copy_real(&(dst->extent), &(src->extent), TRUE) < 0)
+        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCOPY, FAIL, "can't copy extent")
+
+    /* If the selection is 'all', update the number of elements selected in the
+     * destination space */
+    if (H5S_SEL_ALL == H5S_GET_SELECT_TYPE(dst))
+        if (H5S_select_all(dst, FALSE) < 0)
+            HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDELETE, FAIL, "can't change selection")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5S_extent_copy() */
+
+/*-------------------------------------------------------------------------
+ * Function:    H5S__extent_copy_real
+ *
+ * Purpose:     Copies a dataspace extent
+ *
+ * Return:      Non-negative on success/Negative on failure
+ *
+ * Programmer:  Quincey Koziol
+ *              Wednesday, June  3, 1998
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5S__extent_copy_real(H5S_extent_t *dst, const H5S_extent_t *src, hbool_t copy_max)
 {
     unsigned u;
     herr_t   ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_NOAPI(FAIL)
+    FUNC_ENTER_PACKAGE
 
     /* Release destination extent before we copy over it */
-    if (H5S_extent_release(dst) < 0)
+    if (H5S__extent_release(dst) < 0)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace extent")
 
     /* Copy the regular fields */
@@ -520,7 +656,7 @@ H5S_extent_copy(H5S_extent_t *dst, const H5S_extent_t *src, hbool_t copy_max)
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5S_extent_copy() */
+} /* end H5S__extent_copy_real() */
 
 /*-------------------------------------------------------------------------
  * Function:    H5S_copy
@@ -553,7 +689,7 @@ H5S_copy(const H5S_t *src, hbool_t share_selection, hbool_t copy_max)
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
     /* Copy the source dataspace's extent */
-    if (H5S_extent_copy(&(dst->extent), &(src->extent), copy_max) < 0)
+    if (H5S__extent_copy_real(&(dst->extent), &(src->extent), copy_max) < 0)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCOPY, NULL, "can't copy extent")
 
     /* Copy the source dataspace's selection */
@@ -916,7 +1052,7 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5S_write(H5F_t *f, hid_t dxpl_id, H5O_t *oh, unsigned update_flags, H5S_t *ds)
+H5S_write(H5F_t *f, H5O_t *oh, unsigned update_flags, H5S_t *ds)
 {
     herr_t ret_value = SUCCEED; /* Return value */
 
@@ -928,7 +1064,7 @@ H5S_write(H5F_t *f, hid_t dxpl_id, H5O_t *oh, unsigned update_flags, H5S_t *ds)
     HDassert(H5S_GET_EXTENT_TYPE(ds) >= 0);
 
     /* Write the current dataspace extent to the dataspace message */
-    if (H5O_msg_write_oh(f, dxpl_id, oh, H5O_SDSPACE_ID, 0, update_flags, &(ds->extent)) < 0)
+    if (H5O_msg_write_oh(f, oh, H5O_SDSPACE_ID, 0, update_flags, &(ds->extent)) < 0)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "can't update simple dataspace message")
 
 done:
@@ -948,7 +1084,7 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5S_append(H5F_t *f, hid_t dxpl_id, H5O_t *oh, H5S_t *ds)
+H5S_append(H5F_t *f, H5O_t *oh, H5S_t *ds)
 {
     herr_t ret_value = SUCCEED; /* Return value */
 
@@ -960,7 +1096,7 @@ H5S_append(H5F_t *f, hid_t dxpl_id, H5O_t *oh, H5S_t *ds)
     HDassert(H5S_GET_EXTENT_TYPE(ds) >= 0);
 
     /* Add the dataspace message to the object header */
-    if (H5O_msg_append_oh(f, dxpl_id, oh, H5O_SDSPACE_ID, 0, 0, &(ds->extent)) < 0)
+    if (H5O_msg_append_oh(f, oh, H5O_SDSPACE_ID, 0, 0, &(ds->extent)) < 0)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "can't add simple dataspace message")
 
 done:
@@ -982,7 +1118,7 @@ done:
  *-------------------------------------------------------------------------
  */
 H5S_t *
-H5S_read(const H5O_loc_t *loc, hid_t dxpl_id)
+H5S_read(const H5O_loc_t *loc)
 {
     H5S_t *ds        = NULL; /* Dataspace to return */
     H5S_t *ret_value = NULL; /* Return value */
@@ -995,7 +1131,7 @@ H5S_read(const H5O_loc_t *loc, hid_t dxpl_id)
     if (NULL == (ds = H5FL_CALLOC(H5S_t)))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
-    if (H5O_msg_read(loc, H5O_SDSPACE_ID, &(ds->extent), dxpl_id) == NULL)
+    if (NULL == H5O_msg_read(loc, H5O_SDSPACE_ID, &(ds->extent)))
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, NULL, "unable to load dataspace info from dataset header")
 
     /* Default to entire dataspace being selected */
@@ -1015,12 +1151,12 @@ done:
 
 /*--------------------------------------------------------------------------
  NAME
-    H5S_is_simple
+    H5S__is_simple
  PURPOSE
     Check if a dataspace is simple (internal)
  USAGE
-    htri_t H5S_is_simple(sdim)
-        H5S_t *sdim;		IN: Pointer to dataspace object to query
+    htri_t H5S__is_simple(sdim)
+    H5S_t *sdim;        IN: Pointer to dataspace object to query
  RETURNS
     TRUE/FALSE/FAIL
  DESCRIPTION
@@ -1028,11 +1164,11 @@ done:
     has orthogonal, evenly spaced dimensions.
 --------------------------------------------------------------------------*/
 static htri_t
-H5S_is_simple(const H5S_t *sdim)
+H5S__is_simple(const H5S_t *sdim)
 {
     htri_t ret_value = FAIL; /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
+    FUNC_ENTER_STATIC_NOERR
 
     /* Check args and all the boring stuff. */
     HDassert(sdim);
@@ -1042,7 +1178,7 @@ H5S_is_simple(const H5S_t *sdim)
         (H5S_GET_EXTENT_TYPE(sdim) == H5S_SIMPLE || H5S_GET_EXTENT_TYPE(sdim) == H5S_SCALAR) ? TRUE : FALSE;
 
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5S_is_simple() */
+} /* end H5S__is_simple() */
 
 /*--------------------------------------------------------------------------
  NAME
@@ -1068,14 +1204,14 @@ H5Sis_simple(hid_t space_id)
     H5TRACE1("t", "i", space_id);
 
     /* Check args and all the boring stuff. */
-    if ((space = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)) == NULL)
+    if (NULL == (space = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a dataspace")
 
-    ret_value = H5S_is_simple(space);
+    ret_value = H5S__is_simple(space);
 
 done:
     FUNC_LEAVE_API(ret_value)
-}
+} /* end H5Sis_simple() */
 
 /*--------------------------------------------------------------------------
  NAME
@@ -1114,7 +1250,7 @@ H5Sset_extent_simple(hid_t space_id, int rank, const hsize_t dims[/*rank*/], con
     H5TRACE4("e", "iIs*[a1]h*[a1]h", space_id, rank, dims, max);
 
     /* Check args */
-    if ((space = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)) == NULL)
+    if (NULL == (space = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a dataspace")
     if (rank > 0 && dims == NULL)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no dimensions specified")
@@ -1154,20 +1290,20 @@ done:
  *
  *-------------------------------------------------------------------------
  */
-static herr_t
+herr_t
 H5S_set_extent_simple(H5S_t *space, unsigned rank, const hsize_t *dims, const hsize_t *max)
 {
     unsigned u;                   /* Local index variable */
     herr_t   ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Check args */
     HDassert(rank <= H5S_MAX_RANK);
     HDassert(0 == rank || dims);
 
     /* shift out of the previous state to a "simple" dataspace.  */
-    if (H5S_extent_release(&space->extent) < 0)
+    if (H5S__extent_release(&space->extent) < 0)
         HGOTO_ERROR(H5E_RESOURCE, H5E_CANTFREE, FAIL, "failed to release previous dataspace extent")
 
     if (rank == 0) { /* scalar variable */
@@ -1194,13 +1330,11 @@ H5S_set_extent_simple(H5S_t *space, unsigned rank, const hsize_t *dims, const hs
         /* Copy the maximum dimensions if specified. Otherwise, the maximal dimensions are the
          * same as the dimension */
         space->extent.max = (hsize_t *)H5FL_ARR_MALLOC(hsize_t, (size_t)rank);
-        if (max != NULL) {
-            HDmemcpy(space->extent.max, max, sizeof(hsize_t) * rank);
-        }
-        else {
+        if (max != NULL)
+            H5MM_memcpy(space->extent.max, max, sizeof(hsize_t) * rank);
+        else
             for (u = 0; u < space->extent.rank; u++)
                 space->extent.max[u] = dims[u];
-        }
     } /* end else */
 
     /* Selection related cleanup */
@@ -1337,7 +1471,8 @@ herr_t
 H5Sencode(hid_t obj_id, void *buf, size_t *nalloc)
 {
     H5S_t *dspace;
-    herr_t ret_value = SUCCEED;
+    hid_t  temp_fapl_id = H5P_DEFAULT;
+    herr_t ret_value    = SUCCEED;
 
     FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "i*x*z", obj_id, buf, nalloc);
@@ -1346,8 +1481,12 @@ H5Sencode(hid_t obj_id, void *buf, size_t *nalloc)
     if (NULL == (dspace = (H5S_t *)H5I_object_verify(obj_id, H5I_DATASPACE)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
-    if (H5S_encode(dspace, (unsigned char *)buf, nalloc) < 0)
-        HGOTO_ERROR(H5E_DATATYPE, H5E_CANTENCODE, FAIL, "can't encode datatype")
+    /* Verify access property list and set up collective metadata if appropriate */
+    if (H5CX_set_apl(&temp_fapl_id, H5P_CLS_FACC, H5I_INVALID_HID, TRUE) < 0)
+        HGOTO_ERROR(H5E_FILE, H5E_CANTSET, H5I_INVALID_HID, "can't set access property list info")
+
+    if (H5S_encode(dspace, (unsigned char **)&buf, nalloc) < 0)
+        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTENCODE, FAIL, "can't encode dataspace")
 
 done:
     FUNC_LEAVE_API(ret_value)
@@ -1367,8 +1506,8 @@ done:
  *
  *-------------------------------------------------------------------------
  */
-static herr_t
-H5S_encode(H5S_t *obj, unsigned char *buf, size_t *nalloc)
+herr_t
+H5S_encode(H5S_t *obj, unsigned char **p, size_t *nalloc)
 {
     H5F_t   *f = NULL;            /* Fake file structure*/
     size_t   extent_size;         /* Size of serialized dataspace extent */
@@ -1393,28 +1532,31 @@ H5S_encode(H5S_t *obj, unsigned char *buf, size_t *nalloc)
 
     /* Verify the size of buffer.  If it's not big enough, simply return the
      * right size without filling the buffer. */
-    if (!buf || *nalloc < (extent_size + select_size + 1 + 1 + 1 + 4))
+    if (!*p || *nalloc < (extent_size + select_size + 1 + 1 + 1 + 4))
         *nalloc = extent_size + select_size + 1 + 1 + 1 + 4;
     else {
+        unsigned char *pp = (*p); /* Local pointer for decoding */
+
         /* Encode the type of the information */
-        *buf++ = H5O_SDSPACE_ID;
+        *pp++ = H5O_SDSPACE_ID;
 
         /* Encode the version of the dataspace information */
-        *buf++ = H5S_ENCODE_VERSION;
+        *pp++ = H5S_ENCODE_VERSION;
 
         /* Encode the "size of size" information */
-        *buf++ = (unsigned char)H5F_SIZEOF_SIZE(f);
+        *pp++ = (unsigned char)H5F_SIZEOF_SIZE(f);
 
         /* Encode size of extent information. Pointer is actually moved in this macro. */
-        UINT32ENCODE(buf, extent_size);
+        UINT32ENCODE(pp, extent_size);
 
         /* Encode the extent part of dataspace */
-        if (H5O_msg_encode(f, H5O_SDSPACE_ID, TRUE, buf, obj) < 0)
+        if (H5O_msg_encode(f, H5O_SDSPACE_ID, TRUE, pp, obj) < 0)
             HGOTO_ERROR(H5E_DATASPACE, H5E_CANTENCODE, FAIL, "can't encode extent space")
-        buf += extent_size;
+        pp += extent_size;
 
         /* Encode the selection part of dataspace.  */
-        if (H5S_SELECT_SERIALIZE(obj, buf) < 0)
+        *p = pp;
+        if (H5S_SELECT_SERIALIZE(obj, p) < 0)
             HGOTO_ERROR(H5E_DATASPACE, H5E_CANTENCODE, FAIL, "can't encode select space")
     } /* end else */
 
@@ -1453,7 +1595,7 @@ H5Sdecode(const void *buf)
     if (buf == NULL)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, H5I_INVALID_HID, "empty buffer")
 
-    if ((ds = H5S_decode((const unsigned char *)buf)) == NULL)
+    if ((ds = H5S_decode((const unsigned char **)&buf)) == NULL)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDECODE, H5I_INVALID_HID, "can't decode object")
 
     /* Register the type and return the ID */
@@ -1479,50 +1621,50 @@ done:
  *
  *-------------------------------------------------------------------------
  */
-static H5S_t *
-H5S_decode(const unsigned char *buf)
+H5S_t *
+H5S_decode(const unsigned char **p)
 {
-    H5F_t        *f = NULL;         /* Fake file structure*/
-    H5S_t        *ds;               /* Decoded dataspace */
-    H5S_extent_t *extent;           /* Entent of decoded dataspace */
-    size_t        extent_size;      /* size of the extent message*/
-    uint8_t       sizeof_size;      /* 'Size of sizes' for file */
-    H5S_t        *ret_value = NULL; /* Return value */
+    H5F_t               *f = NULL;         /* Fake file structure*/
+    H5S_t               *ds;               /* Decoded dataspace */
+    H5S_extent_t        *extent;           /* Extent of decoded dataspace */
+    const unsigned char *pp = (*p);        /* Local pointer for decoding */
+    size_t               extent_size;      /* size of the extent message*/
+    uint8_t              sizeof_size;      /* 'Size of sizes' for file */
+    H5S_t               *ret_value = NULL; /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT
 
     /* Decode the type of the information */
-    if (*buf++ != H5O_SDSPACE_ID)
+    if (*pp++ != H5O_SDSPACE_ID)
         HGOTO_ERROR(H5E_DATASPACE, H5E_BADMESG, NULL, "not an encoded dataspace")
 
     /* Decode the version of the dataspace information */
-    if (*buf++ != H5S_ENCODE_VERSION)
+    if (*pp++ != H5S_ENCODE_VERSION)
         HGOTO_ERROR(H5E_DATASPACE, H5E_VERSION, NULL, "unknown version of encoded dataspace")
 
     /* Decode the "size of size" information */
-    sizeof_size = *buf++;
+    sizeof_size = *pp++;
 
     /* Allocate "fake" file structure */
     if (NULL == (f = H5F_fake_alloc(sizeof_size)))
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTALLOC, NULL, "can't allocate fake file struct")
 
     /* Decode size of extent information */
-    UINT32DECODE(buf, extent_size);
+    UINT32DECODE(pp, extent_size);
 
     /* Decode the extent part of dataspace */
     /* (pass mostly bogus file pointer and bogus DXPL) */
-    if ((extent = (H5S_extent_t *)H5O_msg_decode(f, H5P_DEFAULT, NULL, H5O_SDSPACE_ID, extent_size, buf)) ==
-        NULL)
+    if (NULL == (extent = (H5S_extent_t *)H5O_msg_decode(f, NULL, H5O_SDSPACE_ID, extent_size, pp)))
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDECODE, NULL, "can't decode object")
-    buf += extent_size;
+    pp += extent_size;
 
     /* Copy the extent into dataspace structure */
-    if ((ds = H5FL_CALLOC(H5S_t)) == NULL)
+    if (NULL == (ds = H5FL_CALLOC(H5S_t)))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL,
                     "memory allocation failed for dataspace conversion path table")
-    if (H5O_msg_copy(H5O_SDSPACE_ID, extent, &(ds->extent)) == NULL)
+    if (NULL == H5O_msg_copy(H5O_SDSPACE_ID, extent, &(ds->extent)))
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCOPY, NULL, "can't copy object")
-    if (H5S_extent_release(extent) < 0)
+    if (H5S__extent_release(extent) < 0)
         HGOTO_ERROR(H5E_RESOURCE, H5E_CANTDELETE, NULL, "can't release previous dataspace")
     extent = H5FL_FREE(H5S_extent_t, extent);
 
@@ -1530,8 +1672,10 @@ H5S_decode(const unsigned char *buf)
     if (H5S_select_all(ds, FALSE) < 0)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTSET, NULL, "unable to set all selection")
 
-    /* Decode the select part of dataspace.  I believe this part always exists. */
-    if (H5S_SELECT_DESERIALIZE(ds, buf) < 0)
+    /* Decode the select part of dataspace.
+     *  Because size of buffer is unknown, assume arbitrarily large buffer to allow decoding. */
+    *p = pp;
+    if (H5S_SELECT_DESERIALIZE(&ds, p, SIZE_MAX) < 0)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDECODE, NULL, "can't decode space selection")
 
     /* Set return value */
@@ -1639,57 +1783,15 @@ H5Sset_extent_none(hid_t space_id)
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a dataspace")
 
     /* Clear the previous extent from the dataspace */
-    if (H5S_extent_release(&space->extent) < 0)
+    if (H5S__extent_release(&space->extent) < 0)
         HGOTO_ERROR(H5E_RESOURCE, H5E_CANTDELETE, FAIL, "can't release previous dataspace")
 
-    space->extent.type = H5S_NO_CLASS;
+    space->extent.type = H5S_NULL;
 
 done:
     FUNC_LEAVE_API(ret_value)
 } /* end H5Sset_extent_none() */
 
-/*--------------------------------------------------------------------------
- NAME
-    H5Soffset_simple
- PURPOSE
-    Changes the offset of a selection within a simple dataspace extent
- USAGE
-    herr_t H5Soffset_simple(space_id, offset)
-        hid_t space_id;	        IN: Dataspace object to reset
-        const hssize_t *offset; IN: Offset to position the selection at
- RETURNS
-    Non-negative on success/Negative on failure
- DESCRIPTION
-        This function creates an offset for the selection within an extent, allowing
-    the same shaped selection to be moved to different locations within a
-    dataspace without requiring it to be re-defined.
---------------------------------------------------------------------------*/
-herr_t
-H5Soffset_simple(hid_t space_id, const hssize_t *offset)
-{
-    H5S_t *space;               /* dataspace to modify */
-    herr_t ret_value = SUCCEED; /* Return value */
-
-    FUNC_ENTER_API(FAIL)
-    H5TRACE2("e", "i*Hs", space_id, offset);
-
-    /* Check args */
-    if (NULL == (space = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
-        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "not a dataspace")
-    if (space->extent.rank == 0 ||
-        (H5S_GET_EXTENT_TYPE(space) == H5S_SCALAR || H5S_GET_EXTENT_TYPE(space) == H5S_NULL))
-        HGOTO_ERROR(H5E_ATOM, H5E_UNSUPPORTED, FAIL, "can't set offset on scalar or null dataspace")
-    if (offset == NULL)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no offset specified")
-
-    /* Set the selection offset */
-    if (H5S_select_offset(space, offset) < 0)
-        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "can't set offset")
-
-done:
-    FUNC_LEAVE_API(ret_value)
-} /* end H5Soffset_simple() */
-
 /*-------------------------------------------------------------------------
  * Function:    H5S_set_extent
  *
@@ -1751,7 +1853,7 @@ done:
  *
  *-------------------------------------------------------------------------
  */
-hbool_t
+H5_ATTR_PURE hbool_t
 H5S_has_extent(const H5S_t *ds)
 {
     hbool_t ret_value = FALSE; /* Return value */
@@ -1860,11 +1962,11 @@ done:
     htri_t H5S_extent_equal(ds1, ds2)
         H5S_t *ds1, *ds2;            IN: Dataspace objects to compare
  RETURNS
-     TRUE if equal, FALSE if unequal on succeess/Negative on failure
+     TRUE if equal, FALSE if unequal on success/Negative on failure
  DESCRIPTION
     Compare two dataspaces if their extents are identical.
 --------------------------------------------------------------------------*/
-htri_t
+H5_ATTR_PURE htri_t
 H5S_extent_equal(const H5S_t *ds1, const H5S_t *ds2)
 {
     unsigned u;                /* Local index variable */
@@ -1923,7 +2025,7 @@ done:
  *
  *-------------------------------------------------------------------------
  */
-hsize_t
+H5_ATTR_PURE hsize_t
 H5S_extent_nelem(const H5S_extent_t *ext)
 {
     FUNC_ENTER_NOAPI_NOINIT_NOERR
@@ -1936,92 +2038,38 @@ H5S_extent_nelem(const H5S_extent_t *ext)
 } /* end H5S_extent_nelem() */
 
 /*-------------------------------------------------------------------------
- * Function:    H5S_set_latest_version
+ * Function:    H5S_set_version
  *
- * Purpose:     Set the encoding for a dataspace to the latest version.
+ * Purpose:     Set the version to encode a dataspace with.
  *
  * Return:      Non-negative on success/Negative on failure
  *
- * Programmer:  Quincey Koziol
- *              Tuesday, July 24, 2007
+ * Programmer:  Vailin Choi; December 2017
  *
  *-------------------------------------------------------------------------
  */
 herr_t
-H5S_set_latest_version(H5S_t *ds)
+H5S_set_version(H5F_t *f, H5S_t *ds)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOERR
-
-    /* Sanity check */
-    HDassert(ds);
-
-    /* Set encoding of extent to latest version */
-    ds->extent.version = H5O_SDSPACE_VERSION_LATEST;
-
-    FUNC_LEAVE_NOAPI(SUCCEED)
-} /* end H5S_set_latest_version() */
-
-#ifndef H5_NO_DEPRECATED_SYMBOLS
-
-/*-------------------------------------------------------------------------
- * Function:	H5S_extend
- *
- * Purpose:	Extend the dimensions of a dataspace.
- *
- * Return:	Success:	Number of dimensions whose size increased.
- *
- *		Failure:	Negative
- *
- * Programmer:	Robb Matzke
- *		Friday, January 30, 1998
- *
- *-------------------------------------------------------------------------
- */
-int
-H5S_extend(H5S_t *space, const hsize_t *size)
-{
-    unsigned u;
-    int      ret_value = 0;
+    unsigned version;             /* Message version */
+    herr_t   ret_value = SUCCEED; /* Return value */
 
     FUNC_ENTER_NOAPI(FAIL)
 
-    /* Check args */
-    HDassert(space && H5S_SIMPLE == H5S_GET_EXTENT_TYPE(space));
-    HDassert(size);
+    /* Sanity check */
+    HDassert(f);
+    HDassert(ds);
 
-    /* Check through all the dimensions to see if modifying the dataspace is allowed */
-    for (u = 0; u < space->extent.rank; u++) {
-        if (space->extent.size[u] < size[u]) {
-            if (space->extent.max && H5S_UNLIMITED != space->extent.max[u] && space->extent.max[u] < size[u])
-                HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "dimension cannot be increased")
-            ret_value++;
-        } /* end if */
-    }     /* end for */
+    /* Upgrade to the version indicated by the file's low bound if higher */
+    version = MAX(ds->extent.version, H5O_sdspace_ver_bounds[H5F_LOW_BOUND(f)]);
 
-    /* Update */
-    if (ret_value) {
-        hsize_t nelem; /* Number of elements in extent */
+    /* Version bounds check */
+    if (version > H5O_sdspace_ver_bounds[H5F_HIGH_BOUND(f)])
+        HGOTO_ERROR(H5E_DATASET, H5E_BADRANGE, FAIL, "Dataspace version out of bounds")
 
-        /* Change the dataspace size & re-compute the number of elements in the extent */
-        for (u = 0, nelem = 1; u < space->extent.rank; u++) {
-            if (space->extent.size[u] < size[u])
-                space->extent.size[u] = size[u];
-
-            nelem *= space->extent.size[u];
-        } /* end for */
-        space->extent.nelem = nelem;
-
-        /* If the selection is 'all', update the number of elements selected */
-        if (H5S_GET_SELECT_TYPE(space) == H5S_SEL_ALL)
-            if (H5S_select_all(space, FALSE) < 0)
-                HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDELETE, FAIL, "can't change selection")
-
-        /* Mark the dataspace as no longer shared if it was before */
-        if (H5O_msg_reset_share(H5O_SDSPACE_ID, space) < 0)
-            HGOTO_ERROR(H5E_DATASPACE, H5E_CANTRESET, FAIL, "can't stop sharing dataspace")
-    } /* end if */
+    /* Set the message version */
+    ds->extent.version = version;
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5S_extend() */
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
+} /* end H5S_set_version() */
