@@ -1,6 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright by The HDF Group.                                               *
- * Copyright by the Board of Trustees of the University of Illinois.         *
  * All rights reserved.                                                      *
  *                                                                           *
  * This file is part of HDF5.  The full HDF5 copyright notice, including     *
@@ -17,34 +16,6 @@
 #ifndef H5public_H
 #define H5public_H
 
-/**\defgroup H5 H5
- *
- * Use the functions in this module to manage the life cycle of HDF5 library
- * instances.
- *
- * <table>
- * <tr><th>Create</th><th>Read</th></tr>
- * <tr valign="top">
- *   <td>
- *   \snippet{lineno} H5_examples.c create
- *   </td>
- *   <td>
- *   \snippet{lineno} H5_examples.c read
- *   </td>
- * <tr><th>Update</th><th>Delete</th></tr>
- * <tr valign="top">
- *   <td>
- *   \snippet{lineno} H5_examples.c update
- *   </td>
- *   <td>
- *   \snippet{lineno} H5_examples.c closing_shop
- *   \snippet{lineno} H5_examples.c delete
- *   </td>
- * </tr>
- * </table>
- *
- */
-
 /* Include files for public use... */
 /*
  * Since H5pubconf.h is a generated header file, it is messy to try
@@ -67,7 +38,7 @@
 #endif
 #ifdef H5_STDC_HEADERS
 #include <limits.h> /* For H5T_NATIVE_CHAR defn in H5Tpublic.h  */
-#include <stdarg.h> /* For variadic functions   */
+#include <stdarg.h> /* For variadic functions in H5VLpublic.h   */
 #endif
 #ifndef __cplusplus
 #ifdef H5_HAVE_STDINT_H
@@ -87,8 +58,12 @@
 
 #ifdef H5_HAVE_PARALLEL
 /* Don't link against MPI C++ bindings */
+#ifndef MPICH_SKIP_MPICXX
 #define MPICH_SKIP_MPICXX 1
-#define OMPI_SKIP_MPICXX  1
+#endif
+#ifndef OMPI_SKIP_MPICXX
+#define OMPI_SKIP_MPICXX 1
+#endif
 #include <mpi.h>
 #ifndef MPI_FILE_NULL /* MPIO may be defined in mpi.h already */
 #include <mpio.h>
@@ -102,24 +77,55 @@
 extern "C" {
 #endif
 
-/* Macros for enabling/disabling particular GCC warnings */
-/* (see the following web-sites for more info:
+/* Macros for enabling/disabling particular GCC / clang warnings
+ *
+ * These are duplicated in H5FDmulti.c (we don't want to put them in the
+ * public header and the multi VFD can't use private headers). If you make
+ * changes here, be sure to update those as well.
+ *
+ * (see the following web-sites for more info:
  *      http://www.dbp-consulting.com/tutorials/SuppressingGCCWarnings.html
  *      http://gcc.gnu.org/onlinedocs/gcc/Diagnostic-Pragmas.html#Diagnostic-Pragmas
  */
-/* These pragmas are only implemented usefully in gcc 4.6+ */
-#if ((__GNUC__ * 100) + __GNUC_MINOR__) >= 406
-#define H5_GCC_DIAG_JOINSTR(x, y) x y
-#define H5_GCC_DIAG_DO_PRAGMA(x)  _Pragma(#x)
-#define H5_GCC_DIAG_PRAGMA(x)     H5_GCC_DIAG_DO_PRAGMA(GCC diagnostic x)
+#define H5_DIAG_JOINSTR(x, y) x y
+#define H5_DIAG_DO_PRAGMA(x)  _Pragma(#x)
+#define H5_DIAG_PRAGMA(x)     H5_DIAG_DO_PRAGMA(GCC diagnostic x)
 
-#define H5_GCC_DIAG_OFF(x) H5_GCC_DIAG_PRAGMA(push) H5_GCC_DIAG_PRAGMA(ignored H5_GCC_DIAG_JOINSTR("-W", x))
-#define H5_GCC_DIAG_ON(x)  H5_GCC_DIAG_PRAGMA(pop)
+#define H5_DIAG_OFF(x) H5_DIAG_PRAGMA(push) H5_DIAG_PRAGMA(ignored H5_DIAG_JOINSTR("-W", x))
+#define H5_DIAG_ON(x)  H5_DIAG_PRAGMA(pop)
+
+/* Macros for enabling/disabling particular GCC-only warnings.
+ * These pragmas are only implemented usefully in gcc 4.6+
+ */
+#if (((__GNUC__ * 100) + __GNUC_MINOR__) >= 406)
+#define H5_GCC_DIAG_OFF(x) H5_DIAG_OFF(x)
+#define H5_GCC_DIAG_ON(x)  H5_DIAG_ON(x)
 #else
 #define H5_GCC_DIAG_OFF(x)
 #define H5_GCC_DIAG_ON(x)
 #endif
 
+/* Macros for enabling/disabling particular clang-only warnings.
+ */
+#if defined(__clang__)
+#define H5_CLANG_DIAG_OFF(x) H5_DIAG_OFF(x)
+#define H5_CLANG_DIAG_ON(x)  H5_DIAG_ON(x)
+#else
+#define H5_CLANG_DIAG_OFF(x)
+#define H5_CLANG_DIAG_ON(x)
+#endif
+
+/* Macros for enabling/disabling particular GCC / clang warnings.
+ * These macros should be used for warnings supported by both gcc and clang.
+ */
+#if (((__GNUC__ * 100) + __GNUC_MINOR__) >= 406) || defined(__clang__)
+#define H5_GCC_CLANG_DIAG_OFF(x) H5_DIAG_OFF(x)
+#define H5_GCC_CLANG_DIAG_ON(x)  H5_DIAG_ON(x)
+#else
+#define H5_GCC_CLANG_DIAG_OFF(x)
+#define H5_GCC_CLANG_DIAG_ON(x)
+#endif
+
 /* Version numbers */
 /**
  * For major interface/format changes
@@ -128,11 +134,11 @@ extern "C" {
 /**
  * For minor interface/format changes
  */
-#define H5_VERS_MINOR 8
+#define H5_VERS_MINOR 10
 /**
  * For tweaks, bug-fixes, or development
  */
-#define H5_VERS_RELEASE 23
+#define H5_VERS_RELEASE 11
 /**
  * For pre-releases like \c snap0. Empty string for official releases.
  */
@@ -140,7 +146,7 @@ extern "C" {
 /**
  * Full version string
  */
-#define H5_VERS_INFO "HDF5 library version: 1.8.23"
+#define H5_VERS_INFO "HDF5 library version: 1.10.11"
 
 #define H5check() H5check_version(H5_VERS_MAJOR, H5_VERS_MINOR, H5_VERS_RELEASE)
 
@@ -247,8 +253,21 @@ typedef int herr_t;
  * }
  * \endcode
  */
-typedef unsigned int hbool_t;
-typedef int          htri_t;
+#ifdef H5_HAVE_STDBOOL_H
+#include <stdbool.h>
+#else /* H5_HAVE_STDBOOL_H */
+#ifndef __cplusplus
+#if defined(H5_SIZEOF_BOOL) && (H5_SIZEOF_BOOL != 0)
+#define bool _Bool
+#else
+#define bool unsigned int
+#endif
+#define true 1
+#define false 0
+#endif /* __cplusplus */
+#endif /* H5_HAVE_STDBOOL_H */
+typedef bool hbool_t;
+typedef int  htri_t;
 
 /* The signed version of size_t
  *
@@ -347,6 +366,10 @@ H5_GCC_DIAG_ON("long-long")
 #error "nothing appropriate for hsize_t"
 #endif
 
+#ifdef H5_HAVE_PARALLEL
+#define HSIZE_AS_MPI_TYPE MPI_UINT64_T
+#endif
+
 /**
  * The address of an object in the file.
  *
@@ -459,6 +482,19 @@ typedef struct H5_ih_info_t {
 } H5_ih_info_t;
 //! <!-- [H5_ih_info_t_snip] -->
 
+/**
+ * Allocation statistics info struct
+ */
+typedef struct H5_alloc_stats_t {
+    unsigned long long total_alloc_bytes;        /**< Running count of total # of bytes allocated */
+    size_t             curr_alloc_bytes;         /**< Current # of bytes allocated */
+    size_t             peak_alloc_bytes;         /**< Peak # of bytes allocated */
+    size_t             max_block_size;           /**< Largest block allocated */
+    size_t             total_alloc_blocks_count; /**< Running count of total # of blocks allocated */
+    size_t             curr_alloc_blocks_count;  /**< Current # of blocks allocated */
+    size_t             peak_alloc_blocks_count;  /**< Peak # of blocks allocated */
+} H5_alloc_stats_t;
+
 /* Functions in H5.c */
 /**
  * \ingroup H5
@@ -581,6 +617,46 @@ H5_DLL herr_t H5garbage_collect(void);
  */
 H5_DLL herr_t H5set_free_list_limits(int reg_global_lim, int reg_list_lim, int arr_global_lim,
                                      int arr_list_lim, int blk_global_lim, int blk_list_lim);
+/**
+ * \ingroup H5
+ * \brief Gets the current size of the free lists used to manage memory
+ *
+ * \param[out] reg_size The current size of all "regular" free list memory used
+ * \param[out] arr_size The current size of all "array" free list memory used
+ * \param[out] blk_size The current size of all "block" free list memory used
+ * \param[out] fac_size The current size of all "factory" free list memory used
+ * \return \herr_t
+ *
+ * \details H5get_free_list_sizes() obtains the current size of the different
+ *          kinds of free lists that the library uses to manage memory. The
+ *          free list sizes can be set with H5set_free_list_limits() and
+ *          garbage collected with H5garbage_collect(). These lists are global
+ *          for the entire library.
+ *
+ * \since 1.10.7
+ */
+H5_DLL herr_t H5get_free_list_sizes(size_t *reg_size, size_t *arr_size, size_t *blk_size, size_t *fac_size);
+/**
+ * \ingroup H5
+ * \brief Gets the memory allocation statistics for the library
+ *
+ * \param[out] stats Memory allocation statistics
+ * \return \herr_t
+ *
+ * \details H5get_alloc_stats() gets the memory allocation statistics for the
+ *          library, if the \c --enable-memory-alloc-sanity-check option was
+ *          given when building the library. Applications can check whether
+ *          this option was enabled detecting if the
+ *          \c H5_MEMORY_ALLOC_SANITY_CHECK macro is defined. This option is
+ *          enabled by default for debug builds of the library and disabled by
+ *          default for non-debug builds. If the option is not enabled, all the
+ *          values returned with be 0. These statistics are global for the
+ *          entire library, but do not include allocations from chunked dataset
+ *          I/O filters or non-native VOL connectors.
+ *
+ * \since 1.10.7
+ */
+H5_DLL herr_t H5get_alloc_stats(H5_alloc_stats_t *stats);
 /**
  * \ingroup H5
  * \brief Returns the HDF library release number
@@ -628,7 +704,7 @@ H5_DLL herr_t H5get_libversion(unsigned *majnum, unsigned *minnum, unsigned *rel
  *          currently linked. If this check fails, H5check_version() causes the
  *          application to abort (by means of a standard C abort() call) and
  *          prints information that is usually useful for debugging. This
- *          precaution is is taken to avoid the risks of data corruption or
+ *          precaution is taken to avoid the risks of data corruption or
  *          segmentation faults.
  *
  *          The most common cause of this failure is that an application was
