@@ -1,6 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Copyright by The HDF Group.                                               *
- * Copyright by the Board of Trustees of the University of Illinois.         *
  * All rights reserved.                                                      *
  *                                                                           *
  * This file is part of HDF5.  The full HDF5 copyright notice, including     *
@@ -47,12 +46,6 @@ H5TOOLS_DLLVAR int         opt_err; /* getoption prints errors if this is on
 H5TOOLS_DLLVAR int         opt_ind; /* token pointer                            */
 H5TOOLS_DLLVAR const char *opt_arg; /* flag argument (or value)                 */
 
-enum {
-    no_arg = 0,  /* doesn't take an argument     */
-    require_arg, /* requires an argument          */
-    optional_arg /* argument is optional         */
-};
-
 /*
  * get_option determines which options are specified on the command line and
  * returns a pointer to any arguments possibly associated with the option in
@@ -88,7 +81,8 @@ typedef struct long_options {
                            * this gets returned from get_option   */
 } long_options;
 
-H5TOOLS_DLL int get_option(int argc, char **argv, const char *opt, const struct long_options *l_opt);
+H5TOOLS_DLL int get_option(int argc, const char *const *argv, const char *opt,
+                           const struct long_options *l_opt);
 /*
  * end get_option section
  */
@@ -119,9 +113,12 @@ typedef struct find_objs_t {
 H5TOOLS_DLLVAR unsigned h5tools_nCols; /*max number of columns for outputting  */
 
 /* Definitions of useful routines */
+H5TOOLS_DLL struct subset_t *parse_subset_params(const char *dset);
+
 H5TOOLS_DLL void   indentation(unsigned);
 H5TOOLS_DLL void   print_version(const char *progname);
 H5TOOLS_DLL void   parallel_print(const char *format, ...);
+H5TOOLS_DLL void   parse_hsize_list(const char *h_list, subset_d *d);
 H5TOOLS_DLL herr_t parse_tuple(const char *start, int sep, char **cpy_out, unsigned *nelems,
                                char ***ptrs_out);
 H5TOOLS_DLL void   error_msg(const char *fmt, ...);
